# The software in this package is distributed under the GNU General
# Public License version 2 (with a special exception described below).
#
# A copy of GNU General Public License (GPL) is included in this distribution,
# in the file COPYING.GPL.
#
# As a special exception, if other files instantiate templates or use macros
# or inline functions from this file, or you compile this file and link it
# with other works to produce a work based on this file, this file
# does not by itself cause the resulting work to be covered
# by the GNU General Public License.
#
# However the source code for this file must still be made available
# in accordance with section (3) of the GNU General Public License.
#
# This exception does not invalidate any other reasons why a work based
# on this file might be covered by the GNU General Public License.
#
# Copyright (c) 2016-2018 Intra2net AG <info@intra2net.com>

""" Helper functions with iterators -- functional program is great!

Currently, recipes from python itertools docu (:py:mod:`itertools`) that have
been deemed useful
"""

import itertools as it
from .type_helpers import PY2, PY3


zip_longest = None
izip_longest = None
if PY2:
    zip_longest = it.izip_longest
    izip_longest = it.izip_longest
else:
    zip_longest = it.zip_longest
    izip_longest = it.zip_longest


def pairwise(iterable):
    """ s -> (s0,s1), (s1,s2), (s2, s3), ...  """
    a, b = it.tee(iterable)
    next(b, None)
    return zip(a, b)


def interesting_repetition(rep):
    """Return True for indices [1,2,3,5,7,10,15,20,30,50,75,100,150,200,...]"""
    if rep in (0, 1, 2, 3, 5, 7):
        return True
    elif rep < 10:
        return False
    elif rep in (10, 15, 20, 30, 50, 75):
        return True
    elif rep < 100:
        return False
    elif rep in (100, 150, 200, 300, 500, 750):
        return True
    elif rep < 1000:
        return False
    elif rep in (1000, 1500, 2000, 3000, 5000, 7500):
        return True
    elif rep < 10000:
        return False
    base = 100
    while True:
        base *= 10
        if rep in (base * num for num in (10, 15, 20, 30, 50, 75)):
            return True
        elif rep < base*100:
            return False

    # could try to do this with math.log (e.g. number of digits is
    # 1+math.floor(math.log(rep, 10))), but quickly get into problems with
    # precision (e.g. log(1000, 10) = 2.9999999999996 should be 3)
